From 7deac183121c507c7c5fb72862aa0c7a1d4a7c89 Mon Sep 17 00:00:00 2001
From: Suhrud Patankar <suhrudpatankar@gmail.com>
Date: Mon, 26 Nov 2018 02:06:02 -0800
Subject: [PATCH] NFSv3 - NLM share reservation access check

The read write of the file is allowed for the owner even if write mode
is not set. For NLM Share reservation, windows client tries to take
write reservation on file with mode as 555.
Allow the owner of the file to take the reservation irrespective of the
mode bits.

Test: git clone from windows client.
git clone https://github.com/apache/geode failes with permission denied

---
 src/SAL/state_share.c |   24 +++++++++++++++++++++++-
 1 files changed, 23 insertions(+), 1 deletions(-)

diff --git a/src/SAL/state_share.c b/src/SAL/state_share.c
index 4041e67..46cac2a 100644
--- a/src/SAL/state_share.c
+++ b/src/SAL/state_share.c
@@ -169,6 +169,7 @@ state_status_t state_nlm_share(struct fsal_obj_handle *obj,
 {
 	fsal_status_t fsal_status = {0, 0};
 	fsal_openflags_t openflags = FSAL_O_CLOSED;
+	fsal_accessflags_t access_mask = 0;
 
 	PTHREAD_MUTEX_lock(&state->state_mutex);
 
@@ -211,10 +212,31 @@ state_status_t state_nlm_share(struct fsal_obj_handle *obj,
 	if (reclaim)
 		openflags |= FSAL_O_RECLAIM;
 
+
+	if (openflags & FSAL_O_READ)
+		access_mask |= FSAL_READ_ACCESS;
+
+	if (openflags & FSAL_O_WRITE)
+		access_mask |= FSAL_WRITE_ACCESS;
+
+	/* The access check must be same as the read and write calls.
+	 * Use owner_skip for the access checks
+	 * The reopen2 call does not use the owner_skip
+	 * perform test_access first and then call reopen2.
+	 */
+	fsal_status = obj->obj_ops.test_access(obj, access_mask,
+                                          NULL, NULL, true);
+        if (FSAL_IS_ERROR(fsal_status)) {
+		LogDebug(COMPONENT_STATE,
+			 "test_access failed with %s",
+			 fsal_err_txt(fsal_status));
+		goto out_unlock;
+        }
+
 	/* Use reopen2 to open or re-open the file and check for share
 	 * conflict.
 	 */
-	fsal_status = fsal_reopen2(obj, state, openflags, true);
+	fsal_status = fsal_reopen2(obj, state, openflags, false);
 
 	if (FSAL_IS_ERROR(fsal_status)) {
 		LogDebug(COMPONENT_STATE,
-- 
1.7.1

